/*
 * fm3_uart.c
 *
 *  Created on: Jan 24, 2012
 *      Author: Kentaro Sekimoto
 */
#include "common.h"
#include "mcu.h"
#include "fm3_uart.h"

#define UART_BUF_SIZE	128

struct FM3_UART_FIFO {
	int ri, wi, ct, run;
	unsigned char buff[UART_BUF_SIZE];
};

#ifndef UART47
FM3_MFS03_UART_TypeDef *UART[] = {
	FM3_MFS0_UART,
	FM3_MFS1_UART,
	FM3_MFS2_UART,
	FM3_MFS3_UART
};
#else
FM3_MFS47_UART_TypeDef *UART[] = {
	FM3_MFS4_UART,
	FM3_MFS5_UART,
	FM3_MFS6_UART,
	FM3_MFS7_UART
};
#endif

volatile struct FM3_UART_FIFO TxFifo, RxFifo;

void delay_ms(volatile unsigned int ms)
{
	ms *= 1000;
	while (ms-- > 0);
}

#ifndef UART47
void FM3_USART_RX(int ch, FM3_MFS03_UART_TypeDef *uart)
#else
void FM3_USART_RX(int ch, FM3_MFS47_UART_TypeDef *uart)
#endif
{
	int i;
	if ((uart->SSR & SSR_RDRF) != 0) {
		unsigned short c = (unsigned short)(uart->RDR);
		if (RxFifo.ct < UART_BUF_SIZE) {
			i = RxFifo.wi;
			RxFifo.buff[i++] = c;
			RxFifo.wi = i % UART_BUF_SIZE;
			RxFifo.ct++;
		}
	} else {
		uart->SSR = uart->SSR | SSR_REC;
	}
}

#ifndef UART47
void FM3_USART_TX(int ch, FM3_MFS03_UART_TypeDef *uart)
#else
void FM3_USART_TX(int ch, FM3_MFS47_UART_TypeDef *uart)
#endif
{
	int i;
	char c;
	if ((uart->SSR & SSR_TDRE) != 0) {
		if (TxFifo.ct) {
			i = TxFifo.ri;
#if 1
			c = TxFifo.buff[i++];
			uart->TDR = (unsigned short)c;
#else
			uart->TDR = TxFifo.buff[i++];
#endif
			TxFifo.ri = i % UART_BUF_SIZE;
			TxFifo.ct--;
		} else {
			TxFifo.run = 0;
			uart->SCR = uart->SCR & ~(SCR_TIE);
		}
	}
}

#ifndef UART47
void FM3_USART_Handle_IRQ(int ch, FM3_MFS03_UART_TypeDef *uart)
#else
void FM3_USART_Handle_IRQ(int ch, FM3_MFS47_UART_TypeDef *uart)
#endif
{
	if ((uart->SSR & (SSR_RDRF | SSR_ORE | SSR_FRE | SSR_PE)) != 0)	{
		FM3_USART_RX(ch, uart);
	} else {
		FM3_USART_TX(ch, uart);
	}
}

#ifdef __GCC__
void MFS0RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS1RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS2RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS3RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS0TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS1TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS2TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS3TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS4RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS5RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS6RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS7RX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS4TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS5TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS6TX_IRQHandler(void *param) __attribute__ ((interrupt));
void MFS7TX_IRQHandler(void *param) __attribute__ ((interrupt));
#endif

#ifndef UART47
void MFS0RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(0, FM3_MFS0_UART);
}

void MFS1RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(1, FM3_MFS1_UART);
}

void MFS2RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(2, FM3_MFS2_UART);
}

void MFS3RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(3, FM3_MFS3_UART);
}

void MFS0TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(0, FM3_MFS0_UART);
}

void MFS1TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(1, FM3_MFS1_UART);
}

void MFS2TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(2, FM3_MFS2_UART);
}

void MFS3TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(3, FM3_MFS3_UART);
}
#else
void MFS4RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(4, FM3_MFS4_UART);
}

void MFS5RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(5, FM3_MFS5_UART);
}

void MFS6RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(6, FM3_MFS6_UART);
}

void MFS7RX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(7, FM3_MFS7_UART);
}

void MFS4TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(4, FM3_MFS4_UART);
}

void MFS5TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(5, FM3_MFS5_UART);
}

void MFS6TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(6, FM3_MFS6_UART);
}

void MFS7TX_IRQHandler(void *param)
{
	FM3_USART_Handle_IRQ(7, FM3_MFS7_UART);
}
#endif

void UARTFIFO_Init(void)
{
	TxFifo.ct = 0;
	TxFifo.ri = 0;
	TxFifo.wi = 0;
	TxFifo.run = 0;
	RxFifo.ct = 0;
	RxFifo.ri = 0;
	RxFifo.wi = 0;
	RxFifo.run = 0;
}

void UART_TxEnable(unsigned int ch)
{
#ifndef UART47
	FM3_MFS03_UART_TypeDef *uart = UART[ch];
#else
	FM3_MFS47_UART_TypeDef *uart = UART[ch-4];
#endif
	uart->SCR |= (SCR_TIE);
}

void UART_TxDisable(unsigned int ch)
{
#ifndef UART47
	FM3_MFS03_UART_TypeDef *uart = UART[ch];
#else
	FM3_MFS47_UART_TypeDef *uart = UART[ch-4];
#endif
	uart->SCR &= ~(SCR_TIE);
}
void UART_RxEnable(unsigned int ch)
{
#ifndef UART47
	FM3_MFS03_UART_TypeDef *uart = UART[ch];
#else
	FM3_MFS47_UART_TypeDef *uart = UART[ch-4];
#endif
	uart->SCR |= (SCR_RIE);
}
void UART_RxDisable(unsigned int ch)
{
#ifndef UART47
	FM3_MFS03_UART_TypeDef *uart = UART[ch];
#else
	FM3_MFS47_UART_TypeDef *uart = UART[ch-4];
#endif
	uart->SCR &= ~(SCR_RIE);
}

void UART_Init(unsigned int ch, unsigned int BaudRate)
{
	unsigned short bgr;
#ifndef UART47
	volatile FM3_MFS03_UART_TypeDef *puart = UART[ch];
#else
	volatile FM3_MFS47_UART_TypeDef *puart = UART[ch-4];
#endif
	switch (ch) {
	case 0:
		FM3_GPIO->ADE &= ~0x80000000;						// Disable AN31
		//FM3_GPIO->ADE = 0;
#if 1
		FM3_GPIO->PFR2 = FM3_GPIO->PFR2 | 0x0006;			// SIN0_0, SOT0_0, SCK0_0
		FM3_GPIO->EPFR07 = FM3_GPIO->EPFR07 | 0x00000050;	// SIN0_0, SOT0_0, SCK0_0
#else
		bFM3_GPIO_PFR2_P1 = 1;
	    bFM3_GPIO_EPFR07_SIN0S0 = 1;
	    bFM3_GPIO_EPFR07_SIN0S1 = 0;
	    bFM3_GPIO_PFR2_P2 = 1;
	    bFM3_GPIO_EPFR07_SOT0B0 = 1;
	    bFM3_GPIO_EPFR07_SOT0B1 = 0;
#endif
	    break;
	case 3:
#if 1
		FM3_GPIO->PFR4 = FM3_GPIO->PFR4 | 0x0300;			// SIN3_2, SOT3_2
		FM3_GPIO->EPFR07 = FM3_GPIO->EPFR07 | 0x03C00000;	// SIN3_2, SOT3_2
#else
		bFM3_GPIO_PFR4_P8 = 1;
		bFM3_GPIO_EPFR07_SIN3S0 = 1;
		bFM3_GPIO_EPFR07_SIN3S1 = 1;
		bFM3_GPIO_PFR4_P9 = 1;
		bFM3_GPIO_EPFR07_SOT3B0 = 1;
		bFM3_GPIO_EPFR07_SOT3B1 = 1;
#endif
	    break;
	case 4:
#if 1
#ifdef MB9BF618T
		FM3_GPIO->PFR0 = FM3_GPIO->PFR0 | 0x0060;			// SIN4_2, SOT4_2, SCK4_2
		FM3_GPIO->EPFR08 = FM3_GPIO->EPFR08 | 0x000000F0;	// SIN4_2, SOT4_2, SCK4_2
#endif
#else
#ifdef MB9BF618T
		bFM3_GPIO_PFR0_P6 = 1;
#else
		bFM3_GPIO_PFR0_PB = 1;
#endif
		bFM3_GPIO_EPFR08_SOT4B0 = 1;
		bFM3_GPIO_EPFR08_SOT4B1 = 1;
#endif
		break;
	default:
		break;
	}
	puart->SMR = (SMR_MD_UART | SMR_SOE);
	puart->SCR = 0x00;
	bgr = (unsigned short)(((SysFrePCLK2 + ((unsigned int)BaudRate / 2)) / (unsigned int)BaudRate) - 1);
	puart->BGR = bgr;
	puart->SCR = puart->SCR & ~SCR_UPGL;
	puart->SSR = 0;
	//puart->BGR = (unsigned short)(((SysFrePCLK2 + ((unsigned int)BaudRate / 2)) / (unsigned int)BaudRate) - 1);
	puart->ESCR = (ESCR_DATABITS_8);
	UARTFIFO_Init();
	NVIC_EnableIRQ((IRQn_Type)(MFS0TX_IRQn + (ch << 1)));
	NVIC_SetPriority((IRQn_Type)(MFS0TX_IRQn + (ch << 1)), FM3_UART_TX_PRIORITY);
	NVIC_EnableIRQ((IRQn_Type)(MFS0RX_IRQn + (ch << 1)));
	NVIC_SetPriority((IRQn_Type)(MFS0RX_IRQn + (ch << 1)), FM3_UART_RX_PRIORITY);
    puart->SCR = puart->SCR | (SCR_RXE | SCR_TXE | SCR_RIE | SCR_TIE);
}

unsigned char UART_Rx(unsigned int ch)
{
	unsigned char c;
	int i;
	//while (!RxFifo.ct) ;
	if (RxFifo.ct) {
	UART_RxDisable(ch);
	i = RxFifo.ri;
	c = RxFifo.buff[i++];
	RxFifo.ri = i % UART_BUF_SIZE;
	RxFifo.ct--;
	UART_RxEnable(ch);
	} else
		c = 0;
	return c;
}

void UART_Tx(unsigned int ch, unsigned char c)
{
	int i;
#ifndef UART47
	volatile FM3_MFS03_UART_TypeDef *puart = UART[ch];
#else
	volatile FM3_MFS47_UART_TypeDef *puart = UART[ch-4];
#endif
	while (TxFifo.ct >= UART_BUF_SIZE) ;
	UART_TxDisable(ch);
	if (TxFifo.run) {
		i = TxFifo.wi;
		TxFifo.buff[i++] = c;
		TxFifo.wi = i % UART_BUF_SIZE;
		TxFifo.ct++;
	} else {
		puart->TDR = c;
		TxFifo.run = 1;
	}
	UART_TxEnable(ch);
}

void UART_TxStr(unsigned int ch, unsigned char *p)
{
	unsigned char c;
	while ((c = *p++) != 0) {
		if (c == '\n')
			UART_Tx(ch, '\r');
		UART_Tx(ch, c);
	}
}
