/*
 * Copyright (c) 2001, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Adam Dunkels <adam@sics.se>
 *
 * $Id: tapdev.c,v 1.8 2006/06/07 08:39:58 adam Exp $
 */

#include "uip.h"
#include "uip_arp.h"
#include "common.h"

/*---------------------------------------------------------------------------*/
void
tapdev_init(void)
{
	u8_t mac[6];
#if defined(LAN8700)
	read_93c46(4, 6, mac);
#elif defined(ICS1894) || defined(LAN8187)
unsigned char MyMACAddr[][6] = {
	MY_MAC_ADDR0,
	MY_MAC_ADDR1
};
#ifdef ETH_CH0
	memcpy(mac, MyMACAddr[0], 6);
#endif
#ifdef ETH_CH1
	memcpy(mac, MyMACAddr[1], 6);
#endif
#endif

	printf("%02x:%02x:%02x:%02x:%02x:%02x\r\n", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	Ether_Init(mac);
	memcpy(&uip_ethaddr, mac, 6);
	return;
}
/*---------------------------------------------------------------------------*/
unsigned int
tapdev_read(void)
{
	unsigned int size;
	EMAC_DMA_RXDESC *prxdesc;
	struct eth_packet *ethp;

	if((prxdesc = RXDESC_Received()) == NULL)
	{
		return 0;
	}
	size = prxdesc->RDES0_f.FL;
	ethp = (struct eth_packet *)prxdesc->RDES2;

	memcpy(uip_buf, ethp, size);
	prxdesc->RDES0_f.OWN = 1;
	return size;
}
/*---------------------------------------------------------------------------*/
void
tapdev_send(void)
{
	Packet_Send(uip_buf, uip_len);
	return;
}
/*---------------------------------------------------------------------------*/
